@@ -38,7 +38,6 @@ Other useful links:
 	http://joda-time.sourceforge.net/key_instant.html
 """
 #===========================================================================
-__version__ = "$Revision: 1.34 $"
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
@@ -57,7 +56,6 @@ from Gnumed.pycommon import gmI18N
 
 
 _log = logging.getLogger('gm.datetime')
-_log.info(__version__)
 _log.info(u'mx.DateTime version: %s', mxDT.__version__)
 
 dst_locally_in_use = None
@@ -239,15 +237,22 @@ def mxdt2py_dt(mxDateTime):
 		)
 		raise
 #===========================================================================
-def format_dob(dob, format='%x', encoding=None, none_string=None):
+def format_dob(dob, format='%Y %b %d', encoding=None, none_string=None, dob_is_estimated=False):
 	if dob is None:
 		if none_string is None:
 			return _('** DOB unknown **')
 		return none_string
 
-	return pydt_strftime(dob, format = format, encoding = encoding, accuracy = acc_days)
+	dob_txt = pydt_strftime(dob, format = format, encoding = encoding, accuracy = acc_days)
+	if dob_is_estimated:
+		return u'%s%s' % (u'\u2248', dob_txt)
+
+	return dob_txt
 #---------------------------------------------------------------------------
-def pydt_strftime(dt, format='%c', encoding=None, accuracy=None):
+def pydt_strftime(dt=None, format='%Y %b %d  %H:%M.%S', encoding=None, accuracy=None):
+
+	if dt is None:
+		dt = pydt_now_here()
 
 	if encoding is None:
 		encoding = gmI18N.get_encoding()
@@ -336,7 +341,7 @@ def py_dt2wxDate(py_dt=None, wx=None):
 #===========================================================================
 # interval related
 #---------------------------------------------------------------------------
-def format_interval(interval=None, accuracy_wanted=None, none_string=None):
+def format_interval(interval=None, accuracy_wanted=None, none_string=None, verbose=False):
 
 	if accuracy_wanted is None:
 		accuracy_wanted = acc_seconds
@@ -355,43 +360,93 @@ def format_interval(interval=None, accuracy_wanted=None, none_string=None):
 	tmp = u''
 
 	if years > 0:
-		tmp += u'%s%s' % (int(years), _('interval_format_tag::years::y')[-1:])
+		if verbose:
+			if years > 1:
+				tag = u' ' + _('years')
+			else:
+				tag = u' ' + _('year')
+		else:
+			tag = _('interval_format_tag::years::y')[-1:]
+		tmp += u'%s%s' % (int(years), tag)
+
 
 	if accuracy_wanted < acc_months:
 		return tmp.strip()
 
 	if months > 0:
-		tmp += u' %s%s' % (int(months), _('interval_format_tag::months::m')[-1:])
+		if verbose:
+			if months > 1:
+				tag = u' ' + _('months')
+			else:
+				tag = u' ' + _('month')
+		else:
+			tag = _('interval_format_tag::months::m')[-1:]
+		tmp += u' %s%s' % (int(months), tag)
 
 	if accuracy_wanted < acc_weeks:
 		return tmp.strip()
 
 	if weeks > 0:
-		tmp += u' %s%s' % (int(weeks), _('interval_format_tag::weeks::w')[-1:])
+		if verbose:
+			if weeks > 1:
+				tag = u' ' + _('weeks')
+			else:
+				tag = u' ' + _('week')
+		else:
+			tag = _('interval_format_tag::weeks::w')[-1:]
+		tmp += u' %s%s' % (int(weeks), tag)
 
 	if accuracy_wanted < acc_days:
 		return tmp.strip()
 
 	if days > 0:
-		tmp += u' %s%s' % (int(days), _('interval_format_tag::days::d')[-1:])
+		if verbose:
+			if days > 1:
+				tag = u' ' + _('days')
+			else:
+				tag = u' ' + _('day')
+		else:
+			tag = _('interval_format_tag::days::d')[-1:]
+		tmp += u' %s%s' % (int(days), tag)
 
 	if accuracy_wanted < acc_hours:
 		return tmp.strip()
 
 	if hours > 0:
-		tmp += u' %s/24' % int(hours)
+		if verbose:
+			if hours > 1:
+				tag = u' ' + _('hours')
+			else:
+				tag = u' ' + _('hour')
+		else:
+			tag = u'/24'
+		tmp += u' %s%s' % (int(hours), tag)
 
 	if accuracy_wanted < acc_minutes:
 		return tmp.strip()
 
 	if mins > 0:
-		tmp += u' %s/60' % int(mins)
+		if verbose:
+			if mins > 1:
+				tag = u' ' + _('minutes')
+			else:
+				tag = u' ' + _('minute')
+		else:
+			tag = u'/60'
+		tmp += u' %s%s' % (int(mins), tag)
 
 	if accuracy_wanted < acc_seconds:
 		return tmp.strip()
 
 	if secs > 0:
-		tmp += u' %s/60' % int(secs)
+		if verbose:
+			if secs > 1:
+				tag = u' ' + _('seconds')
+			else:
+				tag = u' ' + _('second')
+		else:
+			tag = u's'
+		tmp += u' %s%s' % (int(mins), tag)
 
 	return tmp.strip()
 #---------------------------------------------------------------------------
@@ -400,41 +455,39 @@ def format_interval_medically(interval=None):
 
 	This isn't mathematically correct but close enough for display.
 	"""
-	# FIXME: i18n for abbrevs
-
 	# more than 1 year ?
 	if interval.days > 363:
 		years, days = divmod(interval.days, 364)
 		leap_days, tmp = divmod(years, 4)
 		months, day = divmod((days + leap_days), 30.33)
 		if int(months) == 0:
-			return "%sy" % int(years)
-		return "%sy %sm" % (int(years), int(months))
+			return u"%s%s" % (int(years), _('interval_format_tag::years::y')[-1:])
+		return u"%s%s %s%s" % (int(years), _('interval_format_tag::years::y')[-1:], int(months), _('interval_format_tag::months::m')[-1:])
 
 	# more than 30 days / 1 month ?
 	if interval.days > 30:
 		months, days = divmod(interval.days, 30.33)
 		weeks, days = divmod(days, 7)
 		if int(weeks + days) == 0:
-			result = '%smo' % int(months)
+			result = u'%smo' % int(months)
 		else:
-			result = '%sm' % int(months)
+			result = u'%s%s' % (int(months), _('interval_format_tag::months::m')[-1:])
 		if int(weeks) != 0:
-			result += ' %sw' % int(weeks)
+			result += u' %s%s' % (int(weeks), _('interval_format_tag::weeks::w')[-1:])
 		if int(days) != 0:
-			result += ' %sd' % int(days)
+			result += u' %s%s' % (int(days), _('interval_format_tag::days::d')[-1:])
 		return result
 
 	# between 7 and 30 days ?
 	if interval.days > 7:
-		return "%sd" % interval.days
+		return u"%s%s" % (interval.days, _('interval_format_tag::days::d')[-1:])
 
 	# between 1 and 7 days ?
 	if interval.days > 0:
 		hours, seconds = divmod(interval.seconds, 3600)
 		if hours == 0:
-			return '%sd' % interval.days
-		return "%sd (%sh)" % (interval.days, int(hours))
+			return '%s%s' % (interval.days, _('interval_format_tag::days::d')[-1:])
+		return "%s%s (%sh)" % (interval.days, _('interval_format_tag::days::d')[-1:], int(hours))
 
 	# between 5 hours and 1 day
 	if interval.seconds > (5*3600):
@@ -460,12 +513,33 @@ def format_interval_medically(interval=None):
 		return '0:%02d' % int(minutes)
 	return "%s.%ss" % (int(minutes), int(seconds))
 #---------------------------------------------------------------------------
+def is_leap_year(year):
+	# year is multiple of 4 ?
+	div, remainder = divmod(year, 4)
+	# no -> not a leap year
+	if remainder > 0:
+		return False
+
+	# year is a multiple of 100 ?
+	div, remainder = divmod(year, 100)
+	# no -> IS a leap year
+	if remainder > 0:
+		return True
+
+	# year is a multiple of 400 ?
+	div, remainder = divmod(year, 400)
+	# yes -> IS a leap year
+	if remainder == 0:
+		return True
+
+	return False
+#---------------------------------------------------------------------------
 def calculate_apparent_age(start=None, end=None):
 	"""The result of this is a tuple (years, ..., seconds) as one would
-	'expect' a date to look like, that is, simple differences between
-	the fields.
+	'expect' an age to look like, that is, simple differences between
+	the fields:
 
-	No need for 100/400 years leap days rule	because 2000 WAS a leap year.
+		(years, months, days, hours, minutes, seconds)
 
 	This does not take into account time zones which may
 	shift the result by one day.
@@ -482,6 +556,12 @@ def calculate_apparent_age(start=None, end=None):
 	if end == start:
 		return (0, 0, 0, 0, 0, 0)
 
+	# steer clear of leap years
+	if end.month == 2:
+		if end.day == 29:
+			if not is_leap_year(start.year):
+				end = end.replace(day = 28)
+
 	# years
 	years = end.year - start.year
 	end = end.replace(year = start.year)
@@ -2169,7 +2249,7 @@ if __name__ == '__main__':
 			fts.accuracy = accuracy
 			print "  accuracy         : %s (%s)" % (accuracy, _accuracy_strings[accuracy])
 			print "  format_accurately:", fts.format_accurately()
-			print "  strftime()       :", fts.strftime('%c')
+			print "  strftime()       :", fts.strftime('%Y %b %d  %H:%M:%S')
 			print "  print ...        :", fts
 			print "  print '%%s' %% ... : %s" % fts
 			print "  str()            :", str(fts)
@@ -2185,23 +2265,45 @@ if __name__ == '__main__':
 		print "fts.get_pydt():", fts.get_pydt()
 	#-------------------------------------------------
 	def test_calculate_apparent_age():
-		start = pydt_now_here().replace(year = 1974).replace(month = 10).replace(day = 23)
+		# test leap year glitches
+		start = pydt_now_here().replace(year = 2000).replace(month = 2).replace(day = 29)
+		end = pydt_now_here().replace(year = 2012).replace(month = 2).replace(day = 27)
+		print "start is leap year: 29.2.2000"
+		print " ", calculate_apparent_age(start = start, end = end)
+		print " ", format_apparent_age_medically(calculate_apparent_age(start = start))
+
+		start = pydt_now_here().replace(month = 10).replace(day = 23).replace(year = 1974)
+		end = pydt_now_here().replace(year = 2012).replace(month = 2).replace(day = 29)
+		print "end is leap year: 29.2.2012"
+		print " ", calculate_apparent_age(start = start, end = end)
+		print " ", format_apparent_age_medically(calculate_apparent_age(start = start))
+
+		start = pydt_now_here().replace(year = 2000).replace(month = 2).replace(day = 29)
+		end = pydt_now_here().replace(year = 2012).replace(month = 2).replace(day = 29)
+		print "start is leap year: 29.2.2000"
+		print "end is leap year: 29.2.2012"
+		print " ", calculate_apparent_age(start = start, end = end)
+		print " ", format_apparent_age_medically(calculate_apparent_age(start = start))
+
+		print "leap year tests worked"
+
+		start = pydt_now_here().replace(month = 10).replace(day = 23).replace(year = 1974)
 		print calculate_apparent_age(start = start)
 		print format_apparent_age_medically(calculate_apparent_age(start = start))
 
-		start = pydt_now_here().replace(year = 1979).replace(month = 3).replace(day = 13)
+		start = pydt_now_here().replace(month = 3).replace(day = 13).replace(year = 1979)
 		print calculate_apparent_age(start = start)
 		print format_apparent_age_medically(calculate_apparent_age(start = start))
 
-		start = pydt_now_here().replace(year = 1979).replace(month = 2, day = 2)
-		end = pydt_now_here().replace(year = 1979).replace(month = 3).replace(day = 31)
+		start = pydt_now_here().replace(month = 2, day = 2).replace(year = 1979)
+		end = pydt_now_here().replace(month = 3).replace(day = 31).replace(year = 1979)
 		print calculate_apparent_age(start = start, end = end)
 
-		start = pydt_now_here().replace(year = 2009).replace(month = 7, day = 21)
+		start = pydt_now_here().replace(month = 7, day = 21).replace(year = 2009)
 		print format_apparent_age_medically(calculate_apparent_age(start = start))
 
 		print "-------"
-		start = pydt_now_here().replace(year = 2011).replace(month = 1).replace(day = 23).replace(hour = 12).replace(minute = 11)
+		start = pydt_now_here().replace(month = 1).replace(day = 23).replace(hour = 12).replace(minute = 11).replace(year = 2011)
 		print calculate_apparent_age(start = start)
 		print format_apparent_age_medically(calculate_apparent_age(start = start))
 	#-------------------------------------------------
@@ -2231,6 +2333,10 @@ if __name__ == '__main__':
 		print pydt_strftime(dt, accuracy = acc_minutes)
 		print pydt_strftime(dt, accuracy = acc_seconds)
 	#-------------------------------------------------
+	def test_is_leap_year():
+		for year in range(1995, 2017):
+			print year, "leaps:", is_leap_year(year)
+	#-------------------------------------------------
 	# GNUmed libs
 	gmI18N.activate_locale()
 	gmI18N.install_domain('gnumed')
@@ -2244,8 +2350,9 @@ if __name__ == '__main__':
 	#test_str2interval()
 	#test_format_interval()
 	#test_format_interval_medically()
-	test_calculate_apparent_age()
 	#test_str2pydt()
 	#test_pydt_strftime()
+	test_calculate_apparent_age()
+	#test_is_leap_year()
 
 #===========================================================================
